<?php

namespace filebase\data\license;

use wcf\system\exception\PermissionDeniedException;
use wcf\system\exception\UserInputException;
use wcf\system\html\output\HtmlOutputProcessor;
use wcf\system\message\embedded\object\MessageEmbeddedObjectManager;
use wcf\system\request\LinkHandler;
use wcf\system\style\StyleHandler;
use wcf\system\WCF;
use wcf\util\FileUtil;

class UserLicenseAction extends LicenseAction {
	/**
	 * @inheritDoc
	 */
	public function create() {
		if (!isset($this->parameters['data']['isUserLicense'])) $this->parameters['data']['isUserLicense'] = 1;
		if (!isset($this->parameters['data']['userID'])) $this->parameters['data']['userID'] = WCF::getUser()->userID;
		if (!isset($this->parameters['data']['username'])) $this->parameters['data']['username'] = WCF::getUser()->username;
		
		if (!empty($this->parameters['htmlInputProcessor'])) {
			/** @noinspection PhpUndefinedMethodInspection */
			$this->parameters['data']['licenseText'] = $this->parameters['htmlInputProcessor']->getHtml();
		}
		
		$license = parent::create();
		$licenseEditor = new LicenseEditor($license);
		
		// save embedded objects
		if (!empty($this->parameters['htmlInputProcessor'])) {
			/** @noinspection PhpUndefinedMethodInspection */
			$this->parameters['htmlInputProcessor']->setObjectID($license->licenseID);
			if (MessageEmbeddedObjectManager::getInstance()->registerObjects($this->parameters['htmlInputProcessor'])) {
				$licenseEditor->update(['hasEmbeddedObjects' => 1]);
			}
		}
		
		if (!FILEBASE_LICENSE_REDACTOR_ENABLE) {
			$filePath = 'licenses/' . $license->licenseID . '.txt';
			$content = $this->parameters['data']['licenseText'];
		} else {
			$filePath = 'licenses/' . $license->licenseID . '.html';
			
			$content = $this->generate($license, $this->parameters['data']['licenseText']);
		}
		
		$filename = FileUtil::getRealPath(FILEBASE_DIR) . $filePath;
		file_put_contents($filename, $content);
		
		$licenseEditor->update([
			'licensePath' => $filePath,
			'licenseURL' => LinkHandler::getInstance()->getLink(null, ['application' => 'filebase']) . $filePath,
			'isPlainText' => intval(!FILEBASE_LICENSE_REDACTOR_ENABLE)
		]);
		
		return new License($license->licenseID);
	}
	
	/**
	 * @param  \filebase\data\license\License  $license
	 * @param  string                          $text
	 * @return string
	 */
	protected function generate(License $license, $text) {
		$this->loadEmbeddedObjects($license);
		$processor = new HtmlOutputProcessor();
		$processor->process($text, 'de.mysterycode.filebase.userlicense', $license->licenseID);
		
		$wcf = WCF::getTPL()->get('__wcf');
		$icons = [
			'appleTouchIcon' => StyleHandler::getInstance()->getStyle()->getFaviconAppleTouchIcon(),
			'manifest' => StyleHandler::getInstance()->getStyle()->getFaviconManifest(),
			'favicon' => $wcf->getFavicon(),
			'msapplication' => StyleHandler::getInstance()->getStyle()->getFaviconBrowserconfig(),
			'themecolor' => StyleHandler::getInstance()->getStyle()->getVariable('wcfHeaderBackground', true)
		];
		$pagetitle = PAGE_TITLE;
		
		$output = <<<HTML
<!DOCTYPE html>
<html dir="ltr" lang="de">
<head>
	<title>{$license->getTitle()} - {$license->username} - {$pagetitle}</title>
	
	<meta charset="utf-8">user.message.disallowedBBCodes
	<meta name="viewport" content="width=device-width, initial-scale=1">
	<meta name="format-detection" content="telephone=no">
	<meta name="robots" content="noindex,nofollow"><meta property="og:site_name" content="WoltLab Suite">
	
	<!-- Icons -->
	<link rel="apple-touch-icon" sizes="180x180" href="{$icons['appleTouchIcon']}">
	<link rel="manifest" href="{$icons['manifest']}">
	<link rel="shortcut icon" href="{$icons['favicon']}">
	<meta name="msapplication-config" content="{$icons['msapplication']}">
	<meta name="theme-color" content="{$icons['themecolor']}">
</head>
<body>
HTML;

		$output .= $processor->getHtml();
		$output .= <<<HTML
</body></html>
HTML;
		
		return $output;
	}
	
	/**
	 * @inheritDoc
	 */
	public function update() {
		if (!empty($this->parameters['htmlInputProcessor'])) {
			/** @noinspection PhpUndefinedMethodInspection */
			$this->parameters['data']['licenseText'] = $this->parameters['htmlInputProcessor']->getHtml();
		}
		
		parent::update();
		
		foreach ($this->getObjects() as $object) {
			if (!empty($this->parameters['htmlInputProcessor'])) {
				/** @noinspection PhpUndefinedMethodInspection */
				$this->parameters['htmlInputProcessor']->setObjectID($object->licenseID);
				if ($object->hasEmbeddedObjects != MessageEmbeddedObjectManager::getInstance()->registerObjects($this->parameters['htmlInputProcessor'])) {
					$object->update(['hasEmbeddedObjects' => $object->hasEmbeddedObjects ? 0 : 1]);
				}
			}
			
			if (!FILEBASE_LICENSE_REDACTOR_ENABLE) {
				$filePath = 'licenses/' . $object->licenseID . '.txt';
				$content = $this->parameters['data']['licenseText'];
			} else {
				$filePath = 'licenses/' . $object->licenseID . '.html';
				
				$content = $this->generate($object->getDecoratedObject(), $this->parameters['data']['licenseText']);
			}
			
			$filename = FileUtil::getRealPath(FILEBASE_DIR) . $filePath;
			file_put_contents($filename, $content);
		}
	}
	
	/**
	 * @inheritDoc
	 */
	public function validateDelete() {
		// read objects
		if (empty($this->objects)) {
			$this->readObjects();
			
			if (empty($this->objects)) {
				throw new UserInputException('objectIDs');
			}
		}
		
		foreach ($this->objects as $object) {
			// validate permissions
			if ($object->isUserLicense && $object->userID == WCF::getUser()->userID) {
				continue;
			}
			else if (is_array($this->permissionsDelete) && !empty($this->permissionsDelete)) {
				WCF::getSession()->checkPermissions($this->permissionsDelete);
				continue;
			}
			
			throw new PermissionDeniedException();
		}
	}
	
	/**
	 * @inheritDoc
	 */
	public function delete() {
		if (empty($this->objects)) {
			$this->readObjects();
		}
		
		foreach ($this->objects as $object) {
			@unlink(FileUtil::getRealPath(FILEBASE_DIR) . $object->licensePath);
		}
		
		return parent::delete();
	}
	
	public function validateDeleteFile() {
		$this->validateDelete();
	}
	
	/**
	 * @inheritDoc
	 */
	public function deleteFile() {
		if (empty($this->objects)) {
			$this->readObjects();
		}
		
		foreach ($this->objects as $object) {
			@unlink(FileUtil::getRealPath(FILEBASE_DIR) . $object->licensePath);
		}
	}
	
	/**
	 * Loads the embedded objects.
	 */
	public function loadEmbeddedObjects(License &$license) {
		if ($license->hasEmbeddedObjects && !$license->embeddedObjectsLoaded) {
			MessageEmbeddedObjectManager::getInstance()->loadObjects('de.mysterycode.filebase.userlicense', [$license->licenseID]);
			/** @noinspection PhpUndefinedFieldInspection */
			$license->embeddedObjectsLoaded = true;
		}
	}
}
