<?php

namespace filebase\form;

use filebase\acp\form\LicenseAddForm;
use filebase\data\license\UserLicenseAction;
use wcf\form\AbstractForm;
use wcf\system\bbcode\BBCodeHandler;
use wcf\system\exception\UserInputException;
use wcf\system\html\input\HtmlInputProcessor;
use wcf\system\message\censorship\Censorship;
use wcf\system\message\quote\MessageQuoteManager;
use wcf\system\WCF;
use wcf\util\MessageUtil;
use wcf\util\StringUtil;

class UserLicenseAddForm extends LicenseAddForm {
	/**
	 * @inheritDoc
	 */
	public $neededPermissions = ['user.filebase.canAddFile'];
	
	/**
	 * @inheritDoc
	 */
	public $neededModules = ['FILEBASE_ENABLE_LICENSE'];
	
	/**
	 * @var string
	 */
	public $licenseText = '';
	
	/**
	 * @var HtmlInputProcessor
	 */
	public $htmlInputProcessor;
	
	/**
	 * @inheritDoc
	 */
	public $messageObjectType = 'de.mysterycode.filebase.userlicense';
	
	/**
	 * @inheritDoc
	 */
	public $maxTextLength = 100000;
	
	/**
	 * @var string
	 */
	public $disallowedBBCodesPermission;
	
	/**
	 * @inheritDoc
	 */
	public $licenseURL = '#';
	
	/**
	 * @inheritDoc
	 */
	public function readParameters() {
		parent::readParameters();
		
		MessageQuoteManager::getInstance()->readParameters();
	}
	
	/**
	 * @inheritDoc
	 */
	public function readFormParameters() {
		parent::readFormParameters();
		
		if (isset($_POST['licenseText'])) $this->licenseText = StringUtil::trim(MessageUtil::stripCrap($_POST['licenseText']));
		
		MessageQuoteManager::getInstance()->readFormParameters();
	}
	
	/**
	 * @inheritDoc
	 */
	public function validate() {
		parent::validate();
		
		if (FILEBASE_LICENSE_REDACTOR_ENABLE) {
			$this->validateText();
		} else {
			if (empty($this->licenseText)) {
				throw new UserInputException('licenseText');
			}
			
			// search for censored words
			if (ENABLE_CENSORSHIP) {
				$result = Censorship::getInstance()->test($this->licenseText);
				if ($result) {
					WCF::getTPL()->assign('censoredWords', $result);
					throw new UserInputException('licenseText', 'censoredWordsFound');
				}
			}
		}
	}
	
	/**
	 * @inheritDoc
	 */
	public function save() {
		AbstractForm::save();
		
		if (FILEBASE_LICENSE_REDACTOR_ENABLE) $this->licenseText = $this->htmlInputProcessor->getHtml();
		
		// create license
		$this->objectAction = new UserLicenseAction([], 'create', [
			'data' => array_merge($this->additionalFields, [
				'licenseName' => $this->licenseName,
				'licenseText' => $this->licenseText,
				'isPlainText' => intval(!FILEBASE_LICENSE_REDACTOR_ENABLE)
			]),
			'htmlInputProcessor' => $this->htmlInputProcessor
		]);
		$this->objectAction->executeAction();
		$this->saved();
		
		// reset values
		$this->licenseName = $this->licenseURL = $this->licenseText = '';
		$this->htmlInputProcessor = null;
		
		MessageQuoteManager::getInstance()->saved();
		
		// show success message
		WCF::getTPL()->assign('success', true);
	}
	
	/**
	 * @inheritDoc
	 */
	public function assignVariables() {
		parent::assignVariables();
		
		MessageQuoteManager::getInstance()->assignVariables();
		
		WCF::getTPL()->assign('licenseText', $this->licenseText);
	}
	
	/**
	 * Validates the message text.
	 * @see \wcf\form\MessageForm::validateText()
	 */
	protected function validateText() {
		if (empty($this->messageObjectType)) {
			throw new \RuntimeException("Expected non-empty message object type for '".get_class($this)."'");
		}
		
		if (empty($this->licenseText)) {
			throw new UserInputException('licenseText');
		}
		
		if ($this->disallowedBBCodesPermission) {
			BBCodeHandler::getInstance()->setDisallowedBBCodes(explode(',', WCF::getSession()->getPermission($this->disallowedBBCodesPermission)));
		} else if($this->disallowedBBCodesPermission === null) {
			BBCodeHandler::getInstance()->setDisallowedBBCodes(explode(',', WCF::getSession()->getPermission('user.message.disallowedBBCodes')));
		}
		
		$this->htmlInputProcessor = new HtmlInputProcessor();
		$this->htmlInputProcessor->process($this->licenseText, $this->messageObjectType, 0);
		
		// check text length
		if ($this->htmlInputProcessor->appearsToBeEmpty()) {
			throw new UserInputException('licenseText');
		}
		$message = $this->htmlInputProcessor->getTextContent();
		if ($this->maxTextLength != 0 && mb_strlen($message) > $this->maxTextLength) {
			throw new UserInputException('licenseText', 'tooLong');
		}
		
		$disallowedBBCodes = $this->htmlInputProcessor->validate();
		if (!empty($disallowedBBCodes)) {
			WCF::getTPL()->assign('disallowedBBCodes', $disallowedBBCodes);
			throw new UserInputException('licenseText', 'disallowedBBCodes');
		}
		
		// search for censored words
		if (ENABLE_CENSORSHIP) {
			$result = Censorship::getInstance()->test($message);
			if ($result) {
				WCF::getTPL()->assign('censoredWords', $result);
				throw new UserInputException('licenseText', 'censoredWordsFound');
			}
		}
	}
}
